/* BlinkenLightsInteractiveMovieProgram
 * Copyright (C) 2019: Stefan Schuermans <stefan@blinkenarea.org>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a blinkenarea.org project
 */

package org.blinkenarea.Blimp;

import org.blinkenarea.BlinkenLib.*;

/// a template based on list of modifiable pixels and copied pixels
public class FrameTemplatePixelLists implements FrameTemplateIf
{
  /// location and color of a fixed pixel
  protected class FixedPixel {
    public int y, x, c; ///< location
    public byte val; ///< value
    public FixedPixel( int y_, int x_, int c_, byte val_ ) {
      y = y_;
      x = x_;
      c = c_;
      val = val_;
    }
  };

  /// location of a content pixel
  protected class ContentPixel {
    public int y, x, c; ///< source location of content
    public ContentPixel( int y_, int x_, int c_ ) {
      y = y_;
      x = x_;
      c = c_;
    }
  };

  /// a copy of a content pixel
  protected class CopyPixel {
    public int y, x, c; ///< destination position
    public int contentIdx; ///< index of content pixel
    public CopyPixel( int y_, int x_, int c_, int contentIdx_ ) {
      y = y_;
      x = x_;
      c = c_;
      contentIdx = contentIdx_;
    }
  }

  int height, width, channels, maxval;
  /// default color (for fixed pixels), must be of length channels
  byte defColor[];
  /// fixed pixels
  FixedPixel fixed[];
  /// actual content
  ContentPixel content[];
  /// copies of content pixels: copies[i] = { y, x, c, index }
  CopyPixel copies[];
  /// local buffer for content pixel colors (used in apply())
  byte contentBuf[];

  public FrameTemplatePixelLists( int height_, int width_, int channels_,
                                  int maxval_ )
  {
    height = height_;
    width = width_;
    channels = channels_;
    maxval = maxval_;
    defColor = null;
    fixed = null;
    content = null;
    copies = null;
    contentBuf = null;
  }

  protected void setDefColor( byte defColor_[] )
  {
    defColor = defColor_;
  }

  protected void setFixed( FixedPixel fixed_[] )
  {
    fixed = fixed_;
  }

  protected void setContent( ContentPixel content_[] )
  {
    content = content_;
    if( content == null ) {
      contentBuf = null;
    } else {
      contentBuf = new byte [content.length];
    }
  }

  protected void setCopies( CopyPixel copies_[] )
  {
    copies = copies_;
  }

  /// get template format as string
  public String getFormat( )
  {
    return width + "x" + height + "-" + channels + "/" + (maxval + 1);
  }

  public void apply( BlinkenFrame frame )
  {
    // ensure frame has desired size (no-op if size is already equal)
    frame.resize( height, width, channels, maxval );

    // get content pixels
    if( content != null ) {
      for( int i = 0; i < content.length; ++i ) {
        ContentPixel cp = content[i];
        contentBuf[i] = frame.getPixel( cp.y, cp.x, cp.c );
      }
    }

    // set entire frame to default color
    if ( defColor != null && defColor.length >= channels ) {
      for( int y = 0; y < height; ++y ) {
        for( int x = 0; x < width; ++x ) {
          for( int c = 0; c < channels; ++c ) {
            frame.setPixel( y, x, c, defColor[c] );
          }
        }
      }
    }

    // set fixed pixels
    if( fixed != null ) {
      for( int i = 0; i < fixed.length; ++i ) {
        FixedPixel fp = fixed[i];
        frame.setPixel( fp.y, fp.x, fp.c, fp.val );
      }
    }

    // set content pixels
    if( content != null ) {
      for( int i = 0; i < content.length; ++i ) {
        ContentPixel ct = content[i];
        frame.setPixel( ct.y, ct.x, ct.c, contentBuf[i] );
      }
    }

    // set copy pixels
    if( copies != null && content != null ) {
      for( int i = 0; i < copies.length; ++i ) {
        CopyPixel cp = copies[i];
        if( cp.contentIdx < content.length ) {
          frame.setPixel( cp.y, cp.x, cp.c, contentBuf[cp.contentIdx] );
        }
      }
    }
  }
}
